(* #load "graphics.cma" *)
open Graphics

type ('n,'f) ab = (* arbre binaire *)
  | Feuille of 'f
  | Noeud of 'n * ('n,'f) ab * ('n,'f) ab

type ('n,'f) it =
  {frangeg:float list; franged:float list;
   value:'n; filsg:(float*'n, 'f) ab; filsd:(float*'n, 'f) ab}

type ('n,'f) ait =
  | Fait of 'f (*Feuille ait*)
  | Nait of ('n,'f) it

let rec distance franged frangeg overlap delta = match franged, frangeg with
  | [],_ |_,[] -> delta
  | a::p, b::q ->
      let new_overlap = overlap +. a -. b in
      distance p q new_overlap (max new_overlap delta)

let rec frange_long frange1 frange2 decalage = match frange1, frange2 with
  | _,[] -> frange1
  | [],b::q -> decalage +. b :: q
  | a::p, b::q -> a :: frange_long p q (decalage -. a +. b)

let rec cit pas = function
  | Feuille f -> Fait f
  | Noeud(n, a, b) -> match cit pas a, cit pas b with
      | Fait fg, Fait fd ->
          Nait {frangeg=[-.pas]; franged=[pas]; value=n; filsg=Feuille fg; filsd=Feuille fd}
      | Fait f, Nait it ->
          Nait {frangeg = -.pas :: 2.*.pas +. List.hd it.frangeg :: List.tl it.frangeg;
                franged = pas :: it.franged;
                value=n;
                filsg=Feuille f;
                filsd=Noeud ((List.hd it.franged, it.value), it.filsg, it.filsd)}
      | Nait it, Fait f ->
          Nait {frangeg = -.pas :: it.frangeg;
                franged = pas :: -2.*.pas +. List.hd it.franged :: List.tl it.franged;
                value=n;
                filsg=Noeud ((List.hd it.franged, it.value), it.filsg, it.filsd);
                filsd=Feuille f}
      | Nait itg, Nait itd ->
          let d = pas +. 0.5 *. distance itg.franged itd.frangeg 0. 0. in
          Nait {frangeg = -.d :: frange_long itg.frangeg itd.frangeg ( 2.*.d);
                franged =   d :: frange_long itd.franged itg.franged (-2.*.d);
                value=n;
                filsg=Noeud ((List.hd itg.franged, itg.value), itg.filsg, itg.filsd);
                filsd=Noeud ((List.hd itd.franged, itd.value), itd.filsg, itd.filsd)}

let draw_ait x y pv tn tf =
  let rec draw x y = function
    | Feuille f -> tf x y f
    | Noeud ((d,n), fg, fd) ->
        let d = int_of_float d in
        let xfg,xfd,yf = x-d, x+d, y-pv in
        moveto xfg yf; lineto x y; lineto xfd yf;
        tn x y n;
        draw xfg yf fg; draw xfd yf fd;
  in function
    | Fait f -> tf x y f
    | Nait it -> draw x y (Noeud ((List.hd it.franged, it.value), it.filsg, it.filsd))

let drawt_pos x y ph pv tn tf arbre =
  draw_ait x y pv tn tf (cit (float_of_int ph/.2.) arbre)

let draw_tree ph pv tn tf arbre =
  let ait = cit (float_of_int ph /.2.) arbre in
  match ait with
    | Fait f -> tf ph (size_y() - pv) f
    | Nait it ->
        let x,_ = List.fold_left (fun (mini,x) decalage ->
                             let x = x+.decalage in min mini x, x)
                    (0.,0.) it.frangeg in
        draw_ait (int_of_float (-.x) + ph/2) (size_y() - pv) pv tn tf ait

(*exemple*)
let drawt_int t =
  let draw_int x y i =
    moveto x y;
    draw_string (string_of_int i) in
  draw_tree 80 30 draw_int draw_int t

let drawt_points arbre =
  let point = fun x y _ -> fill_circle x y 2 in
  draw_tree 6 6 point point arbre
